package com.sogou.flutter.sg_video_player;

import android.support.annotation.NonNull;
import android.util.Log;
import android.util.LongSparseArray;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.view.TextureRegistry;

/**
 * SgVideoPlayerPlugin
 */
public class SgVideoPlayerPlugin implements FlutterPlugin, MethodCallHandler {
    private static final String PLUGIN_VIDEO_PLAYER = "sogou.flutter/videoPlayer";
    private LongSparseArray<VideoPlayer> videoPlayers = new LongSparseArray();

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        final MethodChannel channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), PLUGIN_VIDEO_PLAYER);
        channel.setMethodCallHandler(this);
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), PLUGIN_VIDEO_PLAYER);
        channel.setMethodCallHandler(new SgVideoPlayerPlugin(registrar));
    }
    /** Register this with the v2 embedding for the plugin to respond to lifecycle callbacks. */
    public SgVideoPlayerPlugin() {}

    private SgVideoPlayerPlugin(Registrar registrar) {
        this.registrar = registrar;
    }

    private PluginRegistry.Registrar registrar;

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        TextureRegistry textures = registrar.textures();
        if (textures == null) {
            result.error("no_activity", "video_player plugin requires a foreground activity", null);
            return;
        }

        switch (call.method) {
            case "init":
                disposeAllPlayers();
                break;
            case "create": {
                TextureRegistry.SurfaceTextureEntry handle = textures.createSurfaceTexture();
                EventChannel eventChannel = new EventChannel(
                        registrar.messenger(), "sogou.flutter/videoPlayer/videoEvents" + handle.id());

                Log.i("SgVideoPlayerPlugin", " Create NEW " + handle.id() + "-->" + call.argument("uri"));

                VideoPlayer player = new VideoPlayer(
                        registrar.context(), eventChannel, handle, result);
                videoPlayers.put(handle.id(), player);
                Log.e("SgVideoPlayerPlugin", "" + handle.id() + "-->" + call.method);

                break;
            }
            default: {
                long textureId = ((Number) call.argument("textureId")).longValue();
                VideoPlayer player = videoPlayers.get(textureId);
                if (player == null) {
                    result.error(
                            "Unknown textureId",
                            "No video player associated with texture id $textureId", null);
                    return;
                }
                onPlayerMethod(call, result, textureId, player);
            }
        }

        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + android.os.Build.VERSION.RELEASE);
        } else {
            result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    }


    private void disposeAllPlayers() {
        for (int i = 0; i < videoPlayers.size(); i++) {
            videoPlayers.valueAt(i).dispose();
        }
        videoPlayers.clear();
    }

    private void onPlayerMethod(MethodCall call, Result result, long textureId, VideoPlayer player) {
        switch (call.method) {
            case "setLooping": {
                player.setLooping((Boolean) call.argument("looping"));
                result.success(null);
            }
            break;
            case "setVolume": {
                player.setVolume((Double) call.argument("volume"));
                result.success(null);
            }
            break;
            case "play": {
                player.play();
                result.success(null);
            }
            break;
            case "pause": {
                player.pause();
                result.success(null);
            }
            break;
            case "seekTo": {
                player.seekTo((Long) call.argument("location"));
                result.success(null);
            }
            break;
            case "position": {
                result.success(player.getPosition());
                player.sendBufferingUpdate();
            }
            break;
            case "dispose": {
                player.dispose();
                videoPlayers.remove(textureId);
                result.success(null);
            }
            break;
            case "reset": {
                // 如果有错误，记得清理错误标记
                String newDataSource = ((String) call.argument("uri"));
                Boolean autoPlay = ((Boolean) call.argument("autoPlay"));

                player.resetToNewDatasource(newDataSource, autoPlay, textureId);
                result.success(null);
            }
            break;
            case "stop": {
                player.stop();
                result.success(null);
            }
            break;
            default:
                result.notImplemented();
        }
    }
}
